#include <utils/ESFile.h>
#include "TransferMgr.hpp"
#include "JpegDecode.hpp"
#include "DTR.hpp"
#include "GrayToMono.hpp"
#include "ColorToGray.hpp"
#include "Lut.hpp"
#include "ProcOrientation.hpp"
#include "MonoToGray.hpp"
#include "Saving.hpp"
#include "Resize.hpp"
#include "GetOrientation.hpp"
#include "Error.hpp"
#include "Padding.hpp"

namespace epsonscan 
{
	int Image::threadCount_ = 0;
	CritSec Image::m_ThreadCriticSec;

	SDIError Image::applyImageProc(std::shared_ptr<KeyMgr> keyMgr, std::shared_ptr<ModelInfo> modelInfo)
	{
			imageProcThread_.reset(new std::thread([this, keyMgr, modelInfo]
			{
				try	
				{
				WaitUntilTheradLimit(keyMgr->GetCommandValueAsNumber(kESXResolution));
				AddThreadCount();

				SDI_TRACE_LOG("applyImageProc start %d", ES_IMAGE_INFO::GetESImagePageCount(imageInfo_));

				imageInfo_[ES_IMAGE_INFO::kESImageInfoResolutionXKey]	 = (ESNumber)keyMgr->GetCommandValueAsNumber(kESXResolution);
				imageInfo_[ES_IMAGE_INFO::kESImageInfoResolutionYKey]	 = (ESNumber)keyMgr->GetCommandValueAsNumber(kESYResolution);
				imageInfo_[ES_IMAGE_INFO::kESImageInfoOutputResolutionKey] = (ESNumber)keyMgr->GetCommandValueAsNumber(kESXResolution);

				JpegDecode jpegDecode(keyMgr, modelInfo);
				ColorToGray colorToGray(keyMgr, modelInfo);
				GrayToMono	grayToMono(keyMgr, modelInfo);
				DTR dtr(keyMgr, modelInfo);
				Lut lut(keyMgr, modelInfo);
				MonoToGray monoToGray(keyMgr, modelInfo);
				Resize resize(keyMgr, modelInfo);
				ProcOrientation procOrientation(keyMgr, modelInfo);
				Saving saving(keyMgr, modelInfo);
				GetOrientation getOrientation(keyMgr, modelInfo);
				Padding padding(keyMgr, modelInfo);

				bool autoJudgedColor = true;
				
				ESNumber whitePageSkipType = kPrvHRD_WhitePageSkipColorType_Gray;
				modelInfo->GetValue(kPrvHRD_WhitePageSkipColorType, whitePageSkipType);

				{
					jpegDecode.Process(imageInfo_, dataBuf_);
				}
				{
					padding.Process(imageInfo_, dataBuf_);
				}
				{	
					if (imageInfo_[ES_IMAGE_INFO::kESImageInfoUpsideDownKey])
					{
						procOrientation.SetOrientation(kSDIOrientation180);
						procOrientation.Process(imageInfo_, dataBuf_);
					}
				}
				{
					dtr.SetActionType(kActionTypeGetDocumentLocation);
					dtr.Process(imageInfo_, dataBuf_);
				}
				{
					resize.Process(imageInfo_, dataBuf_);
				}
				{
					dtr.SetActionType(kActionTypeJudgeColorType);
					dtr.Process(imageInfo_, dataBuf_);
					autoJudgedColor = dtr.IsColorPage();					
				}
				{				
					if (whitePageSkipType == kPrvHRD_WhitePageSkipColorType_Color)
					{
						dtr.SetActionType(kActionTypeJudgeBlankPage);
						dtr.Process(imageInfo_, dataBuf_);
						isBlankPage_ = dtr.IsBlankPage();
						if (isBlankPage_)
						{
							dataBuf_.FreeBuffer();
							SDI_TRACE_LOG("blank page found  %d", ES_IMAGE_INFO::GetESImagePageCount(imageInfo_));
							SubThreadCount();
							return;
						}
					}
				}
				{
					lut.SetAutoJudgedColor(autoJudgedColor);
					lut.Process(imageInfo_, dataBuf_);					
				}
				{
					if (whitePageSkipType == kPrvHRD_WhitePageSkipColorType_Gray)
					{
						dtr.SetActionType(kActionTypeJudgeBlankPage);
						dtr.Process(imageInfo_, dataBuf_);
						isBlankPage_ = dtr.IsBlankPage();
						if (isBlankPage_)
						{
							dataBuf_.FreeBuffer();
							SDI_TRACE_LOG("blank page found  %d", ES_IMAGE_INFO::GetESImagePageCount(imageInfo_));
							SubThreadCount();
							return;
						}
					}
				}
				{
					colorToGray.SetAutoJudgedColor(autoJudgedColor);
					colorToGray.Process(imageInfo_, dataBuf_);
					grayToMono.SetAutoJudgedColor(autoJudgedColor);
					grayToMono.Process(imageInfo_, dataBuf_);					
				}
				{
					if (keyMgr->GetValueInt(kSDIOrientationKey) == kSDIOrientationAuto)
					{
						getOrientation.Process(imageInfo_, dataBuf_);
						if (getOrientation.GetJudgeResult() != kSDIOrientationNone)
						{
							procOrientation.SetOrientation(getOrientation.GetJudgeResult());
							procOrientation.Process(imageInfo_, dataBuf_);
						}
					}
					else
					{	
							procOrientation.SetOrientation((SDIOrientation)keyMgr->GetValueInt(kSDIOrientationKey));
							procOrientation.Process(imageInfo_, dataBuf_);
					}
				}
				{
					monoToGray.Process(imageInfo_, dataBuf_);
				}
				{
					saving.Process(imageInfo_, dataBuf_);
				}

				filePath_ = saving.GetOutputFileName();				
				dataBuf_.FreeBuffer();

				SDI_TRACE_LOG("applyImageProc end %d", ES_IMAGE_INFO::GetESImagePageCount(imageInfo_));

				}catch(std::bad_alloc& memerror)
				{
					SDI_TRACE_LOG("memory  error");
					lastError_ = kSDIErrorNoMemory;	
				}catch(RuntimeError& runtimeError)
				{
					SDI_TRACE_LOG("runtime error = %s", runtimeError.what());
					lastError_ = runtimeError.GetCode();
				}
				catch(...)
				{
					SDI_TRACE_LOG("unknown  error");
					lastError_ = kSDIErrorUnknownError;
				}

				SubThreadCount();
			}));


			return kSDIErrorNone;
	}
}//namespace epsonscan
